@@ -570,24 +570,30 @@ class Shrinker(object):
         """Yield each pair of blocks ``(a, b)``, such that ``a.index <
         b.index``, but only if ``accept_first(a)`` and ``accept_second(b)`` are
         both true."""
-        i = 0
-        while i < len(self.blocks):
-            j = i + 1
-            while j < len(self.blocks):
+
+        # Iteration order here is significant: Rather than fixing i and looping
+        # over each j, then doing the same, etc. we iterate over the gap between
+        # i and j and then over i. The reason for this is that it ensures that
+        # we try a different value for i and j on each iteration of the inner
+        # loop. This stops us from stalling if we happen to hit on a value of i
+        # where nothing useful can be done.
+        #
+        # In the event that nothing works, this doesn't help and we still make
+        # the same number of calls, but by ensuring that we make progress we
+        # have more opportunities to make shrinks that speed up the tests or
+        # that reduce the number of viable shrinks at the next gap size because
+        # we've lowered some values.
+        offset = 1
+        while offset < len(self.blocks):
+            i = 0
+            while i + offset < len(self.blocks):
+                j = i + offset
                 block_i = self.blocks[i]
-                if not accept_first(block_i):
-                    break
                 block_j = self.blocks[j]
-                if not accept_second(block_j):
-                    j += 1
-                    continue
-
-                yield (block_i, block_j)
-                # After this point, the shrink target could have changed,
-                # so blocks need to be re-checked.
-
-                j += 1
-            i += 1
+                if accept_first(block_i) and accept_second(block_j):
+                    yield (block_i, block_j)
+                i += 1
+            offset += 1
 
     def pass_to_descendant(self):
         """Attempt to replace each example with a descendant example.
@@ -1257,7 +1263,15 @@ class Shrinker(object):
         step that gives us a lot of good opportunities to slip to a smaller
         representation of the same bug.
         """
-        for c in range(255, 0, -1):
+
+        # We perform our normalization in a random order. This helps give
+        # us a good mix of likely to succeed (e.g. rare bytes) vs likely
+        # to have a large impact (e.g. common bytes) without having to
+        # have any idea which bytes are which.
+        all_bytes = list(hrange(256))
+        self.random.shuffle(all_bytes)
+
+        for c in all_bytes:
             buf = self.buffer
 
             if c not in buf:
